package cn.com.qmth.scancentral.service.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.scancentral.bean.student.ExamProvinceInfo;
import cn.com.qmth.scancentral.service.DataCacheService;
import cn.com.qmth.scancentral.service.StudentService;

@Service
public class DataCacheServiceImpl implements DataCacheService {

    protected static final Logger log = LoggerFactory.getLogger(DataCacheServiceImpl.class);

    private Map<Long, Set<String>> provinceCache = new ConcurrentHashMap<>();

    private Lock provinceLock = new ReentrantLock();

    @Autowired
    private StudentService studentService;

    @Override
    public void loadCache() {
        loadProvince();
    }

    private void loadProvince() {
        provinceCache.clear();
        List<ExamProvinceInfo> provinces = studentService.listProvince();
        for (ExamProvinceInfo province : provinces) {
            if (StringUtils.isNotBlank(province.getProvince())) {
                addProvinceCache(province.getExamId(), province.getProvince());
            }
        }
    }

    private void addProvinceCache(Long examId, String province) {
        provinceLock.lock();
        try {
            Set<String> cache = provinceCache.get(examId);
            if (cache == null) {
                cache = new HashSet<>();
                provinceCache.put(examId, cache);
            }
            cache.add(province);
        } finally {
            provinceLock.unlock();
        }
    }

    @Override
    public List<String> listProvinceByExamId(Long examId) {
        Set<String> cache = provinceCache.get(examId);
        List<String> ret = new ArrayList<>();
        if (cache == null) {
            return ret;
        }
        ret.addAll(cache);
        Collections.sort(ret);
        return ret;
    }

}
